#!/usr/bin/env python

#############################################################
#                                                           #
# freeleech-dl - what.cd freeleech downloader               #
# written by Hunter Hammond (huntrar@gmail.com)             #
#                                                           #
#############################################################


import argparse as argp
import getpass
import random
import sys
import time

from selenium import webdriver
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.common.keys import Keys

from . import __version__


if sys.version < '3':
    try:
        input = raw_input
    except NameError:
        pass

USER_AGENTS = ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:11.0) '
               'Gecko/20100101 Firefox/11.0',
               'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:22.0) '
               'Gecko/20100 101 Firefox/22.0',
               'Mozilla/5.0 (Windows NT 6.1; rv:11.0) '
               'Gecko/20100101 Firefox/11.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_4) '
               'AppleWebKit/536.5 (KHTML, like Gecko) '
               'Chrome/19.0.1084.46 Safari/536.5',
               'Mozilla/5.0 (Windows; Windows NT 6.1) '
               'AppleWebKit/536.5 (KHTML, like Gecko) '
               'Chrome/19.0.1084.46 Safari/536.5')


def get_parser():
    parser = argp.ArgumentParser(description='what.cd freeleech downloader')
    parser.add_argument('-rt', '--rutorrent', type=str,
                        help='ruTorrent web portal url')
    parser.add_argument('-l', '--local', action='store_true',
                        help='download .torrent files locally instead')
    parser.add_argument('-v', '--version', help='display current version',
                        action='store_true')
    return parser


def get_driver():
    ''' Search PATH for Firefox or Chrome executables (or their variants) '''
    try:
        profile = webdriver.FirefoxProfile()
        profile.set_preference('general.useragent.override\
                               ', random.choice(USER_AGENTS))
        return webdriver.Firefox(profile)
    except WebDriverException:
        try:
            options = webdriver.ChromeOptions()
            options.add_argument('--user-agent={}\
                                 '.format(random.choice(USER_AGENTS)))
            return webdriver.Chrome(chrome_options=options)
        except WebDriverException:
            raise WebDriverException('Failed to find a working browser.')


def what_login():
    user = input('what.cd username: ')
    if not user:
        user = getpass.getuser()

    password = getpass.getpass()

    if user and password:
        url = 'https://what.cd/login.php' 

        driver = get_driver()
        driver.get(url)
        element = driver.find_element_by_xpath('//input[@id="username"]')
        element.send_keys(user)
        element = driver.find_element_by_xpath('//input[@id="password"]')
        element.send_keys(password)
        element.send_keys(Keys.RETURN)

        ''' Give time to login '''
        time.sleep(5)
        return driver
    return False


def get_torrent_links(local=False):
    driver = what_login()
    if driver:
        url = 'https://what.cd/torrents.php?page=1&freetorrent=1'
        driver.get(url)

        ''' Get DL links '''
        elements = driver.find_elements_by_link_text('DL')
        if elements:
            ''' For downloading .torrent files locally '''
            if local:
                return elements

            ''' For getting a list of torrent links to enter elsewhere '''
            return [link.get_attribute('href') for link in elements]
    return False


def main():
    parser = get_parser()
    args = vars(parser.parse_args())

    if args['version']:
        print(__version__)
        return

    if args['rutorrent']:
        rt_url = args['rutorrent']

        ''' Remove scheme if possible '''
        if rt_url.startswith('http://'):
            rt_url = rt_url.replace('http://', '')
        elif rt_url.startswith('https://'):
            rt_url = rt_url.replace('https://', '')

        user = input('ruTorrent username: ')
        if not user:
            user = getpass.getuser()

        password = getpass.getpass()

        torrents = get_torrent_links()
        if torrents and user and password:
            rt_url = 'http://{0}:{1}@{2}'.format(user, password, rt_url)

            driver = get_driver()

            ''' Make WebDriver poll the DOM if elements not immediately found '''
            driver.implicitly_wait(5) # seconds 

            driver.get(rt_url)

            ''' Click the Add Torrent button in ruTorrent web client '''
            element = driver.find_element_by_xpath('//a[@id="mnu_add"]')
            element.click()

            for url in torrents:
                ''' Enter Torrent URL '''
                xpath = '//form[@id="addtorrenturl"]/input[@id="url"]'
                element = driver.find_element_by_xpath(xpath)
                element.send_keys(url)
                element.send_keys(Keys.RETURN)

                print('Added {0} to ruTorrent'.format(url))

                ''' Give time to load the added torrent '''
                time.sleep(5)
    elif args['local']:
        torrents = get_torrent_links(local=True)
        if torrents:
            for url in torrents:
                ''' Click torrent to DL '''
                url.click()

                print('Downloaded {0}'.format(url.get_attribute('href')))
                
                ''' Wait to to allow time to download '''
                time.sleep(3)
    else:
        parser.print_help()
        return



if __name__ == '__main__':
    main()
